# OauthIm
The IIAB apps use an OAuth service provider (currently FusionAuth). This gem
serves to standardize integration with this service. The hope is that,
at some point, we can add this service to the CMS, Kamaji, and related
apps.

## Installation
Add this line to your application's Gemfile:

```ruby
gem 'oauth_im', '0.x.y'   # e.g., '0.1.2'
```

Then run:
```bash
$ bundle
```

## Configuration
Once the gem is installed, add an initializer. Here is an example:

```ruby
# config/initializers/oauth_im.rb
module OauthIm
  configure do |config|
    config.api_key       = ENV['FUSION_AUTH_API_KEY']
    config.callback_path = ENV['FUSION_CALLBACK_PATH'] || DEFAULT_CALLBACK_PATH
    config.client_id     = ENV['FUSION_AUTH_CLIENT_ID']
    config.client_secret = ENV['FUSION_AUTH_CLIENT_SECRET']
    config.domain        = ENV['FUSION_AUTH_DOMAIN']
    config.hmac          = ENV['FUSION_AUTH_HMAC']
    config.iss_domain    = ENV['FUSION_AUTH_ISS_DOMAIN']
    config.tenant_id     = ENV['FUSION_AUTH_TENANT_ID']
    config.authorize_url = ENV['FUSION_AUTH_AUTHORIZE_URL'] || DEFAULT_AUTHORIZE_URL
    config.token_url     = ENV['FUSION_AUTH_TOKEN_URL']     || DEFAULT_TOKEN_URL
  end
end
```

* The `ENV` variable values can be obtained from the OAuth provider.
* The `callback_path` setting is used in two related ways:
  * It [defines a route](https://github.com/illustrativemathematics/oauth_im/blob/main/config/routes.rb#L4) to the [`OAuthIm::ClientController#callback`
    action](https://github.com/illustrativemathematics/oauth_im/blob/main/app/controllers/oauth_im/client_controller.rb#L7-L12).
  * It defines a [callback URL](https://github.com/illustrativemathematics/oauth_im/blob/main/app/controllers/oauth_im/client_controller.rb#L69) used by the OAuth provider.
* Note that this callback URL must be whitelisted at the provider.
  At FusionAuth, this is done under the `Applications|OAuth` tab.
  * For instance, for the app `staging-kh-iiab.herokuapp.com`, if
    `config.callback_path` is set to `callback` (the default), then
    the URL `https://staging-kh-iiab.herokuapp.com/oauth_im/callback`
    must be entered in the OAuth provider's list of authorized
    redirect URLs.

## Usage
### Helpers for Logging in and Out
The engine provides [two endpoints](https://github.com/illustrativemathematics/oauth_im/blob/main/config/routes.rb#L5-L6) for logging in and out, and exposes
corresponding view helpers. These are accessible from the main app as:

| path | url |
|------|-----|
| `oauth_im.login_path` | `oauth_im.login_url` |
| `oauth_im.logout_path` | `oauth_im.logout_url` |

* Note that the helpers are namespaced to the engine.

The [controller actions](https://github.com/illustrativemathematics/oauth_im/blob/main/app/controllers/oauth_im/client_controller.rb#L14-L21) for these routes are provided and should "just
work." Note that there are no view templates associated with these
actions, since requests to them are redirected to the OAuth provider.

### Helpers for User ID and Authentication
The gem provides a controller concern, `OauthIm::Authenticable`, that
exposes [two helper methods](https://github.com/illustrativemathematics/oauth_im/blob/main/app/controllers/concerns/oauth_im/authenticable.rb#L9-L10) for use in views:
* `authenticated?`: returns `true` if the user has been authenticated
  by the OAuth service, false otherwise.
* `email`: returns the current user's authenticated email address.

You can include this concern in your app's `ApplicationController` or
some other appropriate location, e.g.:
``` ruby
class ApplicationController < ActionController::Base
  include OauthIm::Authenticable

  # etc. etc. etc.
end

```

## Gem Maintenance
After many false starts, this repo includes two (seemingly functional) github workflows.

1. Thanks to [this workflow](https://github.com/illustrativemathematics/oauth_im/blob/main/.github/workflows/ci.yml),
   largely copied from the CMS, the specs should run automatically when a PR is issued.
2. Thanks to [this workflow](https://github.com/illustrativemathematics/oauth_im/blob/main/.github/workflows/release.yml)
   based on [this blog post](https://andrewm.codes/blog/automating-ruby-gem-releases-with-github-actions/)
   and using [this release phase action](https://github.com/illustrativemathematics/oauth_im/blob/main/.github/workflows/release.yml),
   certain so-called [conventional commits](https://github.com/google-github-actions/release-please-action#how-should-i-write-my-commits)
   will result in a so-called **release PR**.
   * Per the above documentation, to issue such a commit, use the following format for your commit message:
     * `fix: a comment about a  minor change that corresponds to a SemVar patch`
     * `feat: a comment about a feature change that corresponds to a SemVar minor`
     * `fix!: a comment about a breaking change that corresponds to a Semvar major`
     * `feat!: ditto`
   * Once a release PR is accepted and merged, a new PR is created that does several things:
     * The gem is versioned by updating [the version file](https://github.com/illustrativemathematics/oauth_im/blob/main/lib/oauth_im/version.rb#L4).
     * The new PR also includes auto-generated updates to [CHANGELOG.md](https://github.com/illustrativemathematics/oauth_im/blob/main/CHANGELOG.md).
   * If this new PR is accepted and merged, several other things happened:
     * A new tag and github release are created.
     * The new gem version is automatically pushed up to Rubygems.

### Notes
* Because the above process adds new commits to the repository, you
  should remember to pull them.
* At Rubygems it is difficult to reuse version numbers for gems. For
  this reason, you shouldn't try to force-push updates to gem version
  numbers. Instead, let the automatic process manage versioning for
  you.
